@@ -21,8 +21,8 @@ from gi.repository import Gtk
 
 from gajim.common import app
 
-from gajim.gtk.dialogs import NewConfirmationDialog
-from gajim.gtk.dialogs import DialogButton
+from gajim.gui.dialogs import ConfirmationDialog
+from gajim.gui.dialogs import DialogButton
 from gajim.plugins.plugins_i18n import _
 
 from openpgp.modules.util import Trust
@@ -38,10 +38,10 @@ TRUST_DATA = {
                     'warning-color'),
     Trust.BLIND: ('security-medium-symbolic',
                   _('Blind Trust'),
-                  'openpgp-dark-success-color'),
+                  'encrypted-color'),
     Trust.VERIFIED: ('security-high-symbolic',
                      _('Verified'),
-                     'success-color')
+                     'encrypted-color')
 }
 
 
@@ -120,7 +120,7 @@ class KeyRow(Gtk.ListBoxRow):
             self.key.delete()
             self.destroy()
 
-        NewConfirmationDialog(
+        ConfirmationDialog(
             _('Delete'),
             _('Delete Public Key'),
             _('This will permanently delete this public key'),
@@ -231,7 +231,7 @@ class VerifiedOption(MenuOption):
     type_ = Trust.VERIFIED
     icon = 'security-high-symbolic'
     label = _('Verified')
-    color = 'success-color'
+    color = 'encrypted-color'
 
     def __init__(self):
         MenuOption.__init__(self)
