@@ -14,6 +14,7 @@
 # You should have received a copy of the GNU General Public License
 # along with OpenPGP Gajim Plugin. If not, see <http://www.gnu.org/licenses/>.
 
+import sys
 import time
 import logging
 from pathlib import Path
@@ -21,8 +22,9 @@ from pathlib import Path
 from nbxmpp.namespaces import Namespace
 from nbxmpp import Node
 from nbxmpp import StanzaMalformed
-from nbxmpp.util import is_error_result
 from nbxmpp.structs import StanzaHandler
+from nbxmpp.errors import StanzaError
+from nbxmpp.errors import MalformedStanzaError
 from nbxmpp.modules.openpgp import PGPKeyMetadata
 from nbxmpp.modules.openpgp import parse_signcrypt
 from nbxmpp.modules.openpgp import create_signcrypt_node
@@ -43,7 +45,11 @@ from openpgp.modules.util import DecryptionFailed
 from openpgp.modules.util import prepare_stanza
 from openpgp.modules.key_store import PGPContacts
 from openpgp.backend.sql import Storage
-from openpgp.backend.pygpg import PGPContext
+
+if sys.platform == 'win32':
+    from openpgp.backend.pygpg import PythonGnuPG as PGPBackend
+else:
+    from openpgp.backend.gpgme import GPGME as PGPBackend
 
 
 log = logging.getLogger('gajim.p.openpgp')
@@ -83,9 +89,9 @@ class OpenPGP(BaseModule):
         own_bare_jid = self.own_jid.getBare()
         path = Path(configpaths.get('MY_DATA')) / 'openpgp' / own_bare_jid
         if not path.exists():
-            path.mkdir(parents=True)
+            path.mkdir(mode=0o700, parents=True)
 
-        self._pgp = PGPContext(self.own_jid, path)
+        self._pgp = PGPBackend(self.own_jid, path)
         self._storage = Storage(path)
         self._contacts = PGPContacts(self._pgp, self._storage)
         self._fingerprint, self._date = self.get_own_key_details()
@@ -117,10 +123,13 @@ class OpenPGP(BaseModule):
             callback=self._public_key_received,
             user_data=fingerprint)
 
-    def _public_key_received(self, result, fingerprint):
-        if is_error_result(result):
+    def _public_key_received(self, task):
+        fingerprint = task.get_user_data()
+        try:
+            result = task.finish()
+        except (StanzaError, MalformedStanzaError) as error:
             log.error('%s => Public Key not found: %s',
-                      self._account, result)
+                      self._account, error)
             return
 
         imported_key = self._pgp.import_key(result.key, result.jid)
@@ -152,16 +161,19 @@ class OpenPGP(BaseModule):
             callback=self._keylist_received,
             user_data=jid)
 
-    def _keylist_received(self, result, jid):
-        if is_error_result(result):
+    def _keylist_received(self, task):
+        jid = task.get_user_data()
+        try:
+            keylist = task.finish()
+        except (StanzaError, MalformedStanzaError) as error:
             log.error('%s => Keylist query failed: %s',
-                      self._account, result)
+                      self._account, error)
             if self.own_jid.bareMatch(jid) and self._fingerprint is not None:
                 self.set_keylist()
             return
 
         log.info('Keylist received from %s', jid)
-        self._process_keylist(result, jid)
+        self._process_keylist(keylist, jid)
 
     def _process_keylist(self, keylist, from_jid):
         if not keylist:
@@ -208,15 +220,22 @@ class OpenPGP(BaseModule):
         signcrypt = Node(node=payload)
 
         try:
-            payload, to, timestamp = parse_signcrypt(signcrypt)
+            payload, recipients, _timestamp = parse_signcrypt(signcrypt)
         except StanzaMalformed as error:
             log.warning('Decryption failed: %s', error)
             log.warning(payload)
             return
 
-        if not self.own_jid.bareMatch(to):
+        if not any(map(self.own_jid.bareMatch, recipients)):
             log.warning('to attr not valid')
-            log.warning(payload)
+            log.warning(signcrypt)
+            return
+
+        keys = self._contacts.get_keys(properties.jid.bare)
+        fingerprints = [key.fingerprint for key in keys]
+        if fingerprint not in fingerprints:
+            log.warning('Invalid fingerprint on message: %s', fingerprint)
+            log.warning('Expected: %s', fingerprints)
             return
 
         log.info('Received OpenPGP message from: %s', properties.jid)
@@ -234,7 +253,9 @@ class OpenPGP(BaseModule):
         keys += self._contacts.get_keys(self.own_jid)
         keys += [Key(self._fingerprint, None)]
 
-        payload = create_signcrypt_node(obj.stanza, NOT_ENCRYPTED_TAGS)
+        payload = create_signcrypt_node(obj.stanza,
+                                        [obj.jid],
+                                        NOT_ENCRYPTED_TAGS)
 
         encrypted_payload, error = self._pgp.encrypt(payload, keys)
         if error:
