@@ -29,7 +29,7 @@ from gajim.common import configpaths
 from gajim.common import helpers
 from gajim.common.const import CSSPriority
 
-from gajim.gtk.dialogs import ErrorDialog
+from gajim.gui.dialogs import ErrorDialog
 
 from gajim.plugins import GajimPlugin
 from gajim.plugins.plugins_i18n import _
